<?php
/*
Plugin Name: Related Posts Pro
plugin URI: https://99robots.com/products/related-posts-pro/
Description: Related Posts Pro is the most extensive plugin on the market for displaying your blog's most related posts.
version: 2.0.0
Author: 99 Robots
Author URI: http://99robots.com
License: GPL2
*/

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

/**
 * Global Definitions
 */

/* Item Name */

if (!defined('NNROBOTS_RELATED_POSTS_ITEM_NAME'))
    define('NNROBOTS_RELATED_POSTS_ITEM_NAME', 'Related Posts Pro');

/* Store URL */

if (!defined('NNROBOTS_RELATED_POSTS_STORE_URL'))
    define('NNROBOTS_RELATED_POSTS_STORE_URL', 'https://99robots.com');

/* Minimum PHP Version */

if (!defined('NNROBOTS_RELATED_POSTS_MIN_PHP_VERSION'))
    define('NNROBOTS_RELATED_POSTS_MIN_PHP_VERSION', '5.4.0');

/* Plugin Name */

if (!defined('NNROBOTS_RELATED_POSTS_PLUGIN_NAME'))
    define('NNROBOTS_RELATED_POSTS_PLUGIN_NAME', trim(dirname(plugin_basename(__FILE__)), '/'));

/* Plugin directory */

if (!defined('NNROBOTS_RELATED_POSTS_PLUGIN_DIR'))
    define('NNROBOTS_RELATED_POSTS_PLUGIN_DIR', WP_PLUGIN_DIR . '/' . NNROBOTS_RELATED_POSTS_PLUGIN_NAME);

/* Plugin url */

if (!defined('NNROBOTS_RELATED_POSTS_PLUGIN_URL'))
    define('NNROBOTS_RELATED_POSTS_PLUGIN_URL', WP_PLUGIN_URL . '/' . NNROBOTS_RELATED_POSTS_PLUGIN_NAME);

/* Plugin verison */

if (!defined('NNROBOTS_RELATED_POSTS_VERSION_NUM'))
    define('NNROBOTS_RELATED_POSTS_VERSION_NUM', '2.0.0');

// Required Files

require_once('core/functions.php');
require_once('admin/functions.php');
require_once('widget.php');
require_once('shortcode.php');

// Hooks / Filters

add_shortcode('nnr_related_posts', 							'nnr_related_posts_shortcode');
register_activation_hook( __FILE__, 						array('NNRobots_Related_Posts', 'register_activation'));
add_action( 'wpmu_new_blog', 								array('NNRobots_Related_Posts', 'activate_new_site'));
add_action( 'init', 										array('NNRobots_Related_Posts', 'load_textdomain'));
add_action( 'admin_menu', 									array('NNRobots_Related_Posts', 'menu_page'));
$plugin = plugin_basename(__FILE__);
add_filter("plugin_action_links_$plugin", 					array('NNRobots_Related_Posts', 'settings_link'));

// Software Licensing Updates

add_action('wp_ajax_nnr_rlp_license_action', array('NNRobots_Related_Posts', 'license_action'));

if ( !class_exists('EDD_SL_Plugin_Updater') ) {
	include_once( dirname( __FILE__ ) . '/include/edd-software-licensing-updates/EDD_SL_Plugin_Updater.php' );
}

$edd_updater = new EDD_SL_Plugin_Updater( NNROBOTS_RELATED_POSTS_STORE_URL , __FILE__, array(
		'version' 	=> NNROBOTS_RELATED_POSTS_VERSION_NUM,
		'license' 	=> NNRobots_Related_Posts::get_license_key(),
		'item_name' => NNROBOTS_RELATED_POSTS_ITEM_NAME,
		'author' 	=> '99 Robots',
	)
);

/**
 *  NNRobots_Related_Posts main class
 *
 * @since 1.0.0
 * @using Wordpress 3.8
 */

class NNRobots_Related_Posts {

	/**
	 * text_domain
	 *
	 * (default value: 'nnrobots-related-posts')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	static $text_domain = 'nnrobots-related-posts';

	/**
	 * prefix
	 *
	 * (default value: 'nnrobots_related_posts_')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	static $prefix = 'nnrobots_related_posts_';

	/**
	 * prefix_dash
	 *
	 * (default value: 'nnrobots_related_posts_')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	static $prefix_dash = 'nnr-rlp-';

	/**
	 * settings_page
	 *
	 * (default value: 'nnrobots-related-posts-settings')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	static $settings_page = 'nnrobots-related-posts-settings';

	/**
	 * default
	 *
	 * @var mixed
	 * @access private
	 * @static
	 */
	static $default = array(
		'title'							=> 'Related Posts',
		'display_in'					=> 'post',
		'location'						=> 'bottom',
		'num_of_columns'                => '2',
		'related_by'					=> 'category',
		'default_thumbnail'             => '',
		'layout'                        => 'list',
        'numbered_list_style'           => 'normal',
        'numbered_list_color'           => '#FFFFFF',
        'numbered_list_circle_color'    => '#94CD4B',
        'grid_size'                     => 'medium',
        'show_comments'					=> false,
        'show_nav'						=> true,
        'show_category'                 => false,
        'show_author'                   => false,
        'show_post_date'                => false,
        'color'                         => '#D3D3D3',
        'text_color'                    => '#000000',
        'fancy_colors'                  => '#D14841,#F37934,#FAC51C,#61BD6D,#2C82C9,#553982',
        'num_of_posts'                  => '6',
        'add_recent'                  	=> false,
        'time_range'					=> 'all_time',
        'order'							=> 'ASC',
        'order_by'						=> 'date',
        'async'                         => false,
        'style_override'                => true,
		'trigger'						=> array(
			'post'	=> array(
				'all'	=> true
			)
		),
		'query'							=> array(
			'post_type'			=> 'post,page',
		),
	);

	/**
	 * Load the text domain
	 *
	 * @since 1.0.0
	 */
	static function load_textdomain() {
		load_plugin_textdomain(self::$text_domain, false, NNROBOTS_RELATED_POSTS_PLUGIN_DIR . '/languages');
	}

	/**
	 * Hooks to 'register_activation_hook'
	 *
	 * @since 1.0.0
	 */
	static function register_activation( $network_wide ) {

		global $wpdb;

	    if ( is_multisite() && $network_wide ) {

	        // Get all blogs in the network and activate plugin on each one
	        $blog_ids = $wpdb->get_col( "
	        	SELECT blog_id FROM $wpdb->blogs
				WHERE archived = '0' AND spam = '0' AND deleted = '0'
			" );
	        foreach ( $blog_ids as $blog_id ) {
	        	switch_to_blog( $blog_id );
				self::single_activate();
	        }

	        // Switch back to the current blog
	        restore_current_blog();

	    } else {
	        self::single_activate();
	    }

		// Check if multisite, if so then save as site option

		if (function_exists("is_multisite") && is_multisite()) {
			add_site_option(self::$prefix . 'version', NNROBOTS_RELATED_POSTS_VERSION_NUM);
		} else {
			add_option(self::$prefix . 'version', NNROBOTS_RELATED_POSTS_VERSION_NUM);
		}
	}

	/**
	 * Fired for each blog when the plugin is activated.
	 *
	 * @since 2.0.0
	 *
	 */
	static function single_activate() {
		global $wpdb;

	    $wpdb->hide_errors();

		// If we're running mySQL v5.6, convert the WPDB posts table to InnoDB, since InnoDB supports FULLTEXT from v5.6 onwards
		if ( version_compare( 5.6, $wpdb->db_version(), '<=' ) ) {
			$wpdb->query( 'ALTER TABLE ' . $wpdb->posts . ' ENGINE = InnoDB;' );
		} else {
			$wpdb->query( 'ALTER TABLE ' . $wpdb->posts . ' ENGINE = MYISAM;' );
		}

		$wpdb->query( 'ALTER TABLE ' . $wpdb->posts . ' ADD FULLTEXT nnr_related_title (post_title);' );
	    $wpdb->show_errors();

	}


	/**
	 * Fired when a new site is activated with a WPMU environment.
	 *
	 * @since 2.0.0
	 *
	 * @param    int    $blog_id    ID of the new blog.
	 */
	static function activate_new_site( $blog_id ) {

		if ( 1 !== did_action( 'wpmu_new_blog' ) ) {
			return;
		}

		switch_to_blog( $blog_id );
		self::single_activate();
		restore_current_blog();

	}

	/**
	 * Hooks to 'plugin_action_links_' filter
	 *
	 * @since 1.0.0
	 */
	static function settings_link($links) {

		$widget_link = '<a href="widgets.php">Widget</a>';
		array_unshift($links, $widget_link);

		$settings_link = '<a href="options-general.php?page=' . self::$settings_page . '">Settings</a>';
		array_unshift($links, $settings_link);

		return $links;
	}

	/**
	 * Perform all the actions with a license
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function license_action() {

		if ( !isset($_POST['license_key']) ) {
			echo '';
			die();
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> $_POST['license_action'],
			'license' 		=> $_POST['license_key'],
			'item_name' 	=> urlencode( NNROBOTS_RELATED_POSTS_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( NNROBOTS_RELATED_POSTS_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			echo $response;
			die();
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		if ( $_POST['license_action'] == 'activate_license' ) {
			self::update_license_key($_POST['license_key']);
		} else if ( $_POST['license_action'] == 'deactivate_license' ) {
			self::update_license_key('');
		}

		// decode the license data

		echo $license_data->license;

		die();

	}

	/**
	 * Check if license key is active
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function is_license_active( $license_key ) {

		if ( !isset( $license_key ) ) {
			return false;
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> 'check_license',
			'license' 		=> $license_key,
			'item_name' 	=> urlencode( NNROBOTS_RELATED_POSTS_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( NNROBOTS_RELATED_POSTS_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			return false;
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// decode the license data

		if ( isset($license_data->license) && $license_data->license == 'valid' ) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Get the license key
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function get_license_key() {

		// Get the license key from based on WordPress install

		if ( function_exists('is_multisite') && is_multisite() ) {
			$license_key = get_site_option(self::$prefix . 'license_key');
		} else {
			$license_key = get_option(self::$prefix . 'license_key');
		}

		// Check if license key is set

		if ( $license_key === false ) {
			$license_key = '';
		}

		return $license_key;
	}

	/**
	 * Update the license key
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function update_license_key($license_key) {

		if ( isset( $license_key ) ) {

			// Get the license key from based on WordPress install

			if ( function_exists('is_multisite') && is_multisite() ) {

				update_site_option(self::$prefix . 'license_key', $license_key);

			} else {

				update_option(self::$prefix . 'license_key', $license_key);

			}
		}
	}

	/**
	 * Hooks to 'admin_menu'
	 *
	 * @since 1.0.0
	 */
	static function menu_page() {

	    // Cast the first sub menu to the top menu

	    $settings_page_load = add_submenu_page(
	    	'options-general.php', 										// parent slug
	    	__('Related Posts Pro', self::$text_domain), 				// Page title
	    	__('Related Posts Pro', self::$text_domain), 				// Menu name
	    	'manage_options', 											// Capabilities
	    	self::$settings_page, 										// slug
	    	array('NNRobots_Related_Posts', 'admin_settings')			// Callback function
	    );
	    add_action("admin_print_scripts-$settings_page_load", array('NNRobots_Related_Posts', 'include_admin_scripts'));
	}

	/**
	 * Hooks to 'admin_print_scripts-$page'
	 *
	 * @since 1.0.0
	 */
	static function include_admin_scripts() {

		// Styles

		wp_enqueue_style('wp-color-picker');
		wp_enqueue_style(self::$prefix . 'bootstrap_css', 		NNROBOTS_RELATED_POSTS_PLUGIN_URL . '/admin/css/nnr-bootstrap.min.css');
		wp_enqueue_style(self::$prefix . 'settings_css', 		NNROBOTS_RELATED_POSTS_PLUGIN_URL . '/admin/css/settings.css');
		wp_enqueue_style(self::$prefix . 'tabs_css', 			NNROBOTS_RELATED_POSTS_PLUGIN_URL . '/admin/css/tabs.css');
		wp_enqueue_style(self::$prefix_dash . 'selectize_css',	NNROBOTS_RELATED_POSTS_PLUGIN_URL . '/admin/css/selectize.bootstrap3.css');
		wp_enqueue_style(self::$prefix_dash . 'fontawesome_css',NNROBOTS_RELATED_POSTS_PLUGIN_URL . '/include/fontawesome/css/font-awesome.min.css');

		// Scripts

		wp_enqueue_media();
		wp_enqueue_script(self::$prefix_dash . 'bootstrap_js', 	NNROBOTS_RELATED_POSTS_PLUGIN_URL . '/admin/js/bootstrap.min.js', array('jquery'));
		wp_enqueue_script(self::$prefix_dash . 'selectize_js', 	NNROBOTS_RELATED_POSTS_PLUGIN_URL . '/admin/js/selectize.min.js', array('jquery'));
		wp_enqueue_script(self::$prefix . 'settings_js', 		NNROBOTS_RELATED_POSTS_PLUGIN_URL . '/admin/js/settings.js', array( 'wp-color-picker', 'jquery', 'jquery-ui-core', 'jquery-ui-sortable' ), false, true);

		$post_types = get_post_types(array('public' => true));
		unset($post_types['attachment']);

		wp_localize_script(self::$prefix . 'settings_js', 		'nnr_rlp_data', array(
			'prefix_dash'		=> self::$prefix_dash,
			'plugins_url'		=> plugins_url('img/layouts/', __FILE__),
			'ajaxurl'			=> admin_url( 'admin-ajax.php' ),
			'admin_url'			=> admin_url(),
			'post_types'		=> $post_types,
			'activate_text'		=> __('Activate', self::$text_domain),
			'deactivate_text'	=> __('Deactivate', self::$text_domain),
			'no_license_key'		=> __('Please add or activate your License Key to get automatic updates.  Without a valid license key you will not receive regular updates. You can find your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>',
			'expired'				=> __('EXPIRED: This license key has expired.  Please renew your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>',
			'activation_error'		=> __('INVALID LICENSE KEY: The license key is not valid please try again. You can find your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>.',
			'activation_valid'		=> __('SUCCESS: Your license key is valid.', self::$text_domain),
			'deactivation_valid'	=> __('SUCCESS: This site has been deactivated.', self::$text_domain),
			'deactivation_error'	=> __('DEACTIVATION FAILED: This site could not be deactivated.', self::$text_domain),
		));
	}

	/**
	 * Displays the HTML for the 'nnrobots-popular-posts-admin-menu-settings' admin page
	 *
	 * @since 1.0.0
	 */
	static function admin_settings() {

		global $wp_version;

		$license_key = self::get_license_key();

		$settings = get_option(self::$prefix . 'settings');

		// Default values

		if ($settings === false) {
			$settings = self::$default;
		}

		$post_types = get_post_types(array('public' => true));
		unset($post_types['attachment']);

		$taxonomies = get_taxonomies(array('public' => true));
		unset($taxonomies['post_format']);

		// Save data nd check nonce

		if (isset($_POST['submit']) && check_admin_referer(self::$prefix . 'admin_settings')) {

			$settings = array(
				'title'							=> isset($_POST[self::$prefix_dash . 'title']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'title'])) : self::$default['title'],
				'display_in'					=> isset($_POST[self::$prefix_dash . 'display-in']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'display-in'])) : self::$default['display_in'],
				'location'						=> isset($_POST[self::$prefix_dash . 'location']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'location'])) : self::$default['location'],
				'num_of_columns'				=> isset($_POST[self::$prefix_dash . 'num-of-columns']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'num-of-columns'])) : self::$default['num_of_columns'],
				'related_by'					=> isset($_POST[self::$prefix_dash . 'related-by']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'related-by'])) : self::$default['related_by'],
				'default_thumbnail'             => isset($_POST[self::$prefix_dash . 'default-thumbnail']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'default-thumbnail'])) : self::$default['default_thumbnail'],
				'layout'                        => isset($_POST[self::$prefix_dash . 'layout']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'layout'])) : self::$default['layout'],
		        'numbered_list_style'           => isset($_POST[self::$prefix_dash . 'numbered-list-style']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'numbered-list-style'])) :
												   self::$default['numbered_list_style'],
		        'numbered_list_color'           => isset($_POST[self::$prefix_dash . 'numbered-list-color']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'numbered-list-color'])) :
												   self::$default['numbered_list_color'],
		        'numbered_list_circle_color'    => isset($_POST[self::$prefix_dash . 'numbered-list-circle-color']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'numbered-list-circle-color'])) :
												   self::$default['numbered_list_circle_color'],
		        'grid_size'                     => isset($_POST[self::$prefix_dash . 'grid-size']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'grid-size'])) : self::$default['grid_size'],
		        'show_comments'					=> isset($_POST[self::$prefix_dash . 'show-comments']) && $_POST[self::$prefix_dash . 'show-comments'] ? true : false,
		        'show_nav'						=> isset($_POST[self::$prefix_dash . 'show-nav']) && $_POST[self::$prefix_dash . 'show-nav'] ? true : false,
		        'show_category'                 => isset($_POST[self::$prefix_dash . 'show-category']) && $_POST[self::$prefix_dash . 'show-category'] ? true : false,
		        'show_author'                   => isset($_POST[self::$prefix_dash . 'show-author']) && $_POST[self::$prefix_dash . 'show-author'] ? true : false,
		        'show_post_date'                => isset($_POST[self::$prefix_dash . 'show-post-date']) && $_POST[self::$prefix_dash . 'show-post-date'] ? true : false,
		        'color'                         => isset($_POST[self::$prefix_dash . 'color']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'color'])) : self::$default['color'],
		        'text_color'                    => isset($_POST[self::$prefix_dash . 'text-color']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'text-color'])) : self::$default['text_color'],
		        'fancy_colors'                  => isset($_POST[self::$prefix_dash . 'fancy-colors']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'fancy-colors'])) : self::$default['fancy_colors'],
				'time_range'                  	=> isset($_POST[self::$prefix_dash . 'time-range']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'time-range'])) : self::$default['time_range'],
		        'num_of_posts'                  => isset($_POST[self::$prefix_dash . 'num-of-posts']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'num-of-posts'])) : self::$default['num_of_posts'],
				'add_recent'                	=> isset($_POST[self::$prefix_dash . 'add-recent']) && $_POST[self::$prefix_dash . 'add-recent'] ? true : false,
				'order'                  	    => isset($_POST[self::$prefix_dash . 'order']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'order'])) : self::$default['order'],
				'order_by'                  	=> isset($_POST[self::$prefix_dash . 'order-by']) ?
												   stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'order-by'])) : self::$default['order_by'],
		        'async'                         => isset($_POST[self::$prefix_dash . 'async']) && $_POST[self::$prefix_dash . 'async'] ? true : false,
		        'style_override'                => isset($_POST[self::$prefix_dash . 'style-override']) && $_POST[self::$prefix_dash . 'style-override'] ? true : false,
			);

			// Post Types

			foreach ($post_types as $post_type) {
				$settings['trigger'][$post_type]['exclude'] = isset($_POST[self::$prefix_dash . 'trigger-exclude-' . $post_type]) ?
													      	  str_replace(' ', '', rtrim(sanitize_text_field($_POST[self::$prefix_dash . 'trigger-exclude-' . $post_type]), ',')) : '';

				$settings['query'][$post_type]['exclude'] = isset($_POST[self::$prefix_dash . 'query-exclude-' . $post_type]) ?
													      	str_replace(' ', '', rtrim(sanitize_text_field($_POST[self::$prefix_dash . 'query-exclude-' . $post_type]), ',')) : '';
			}

			$settings['query']['post_type']	  = isset($_POST[self::$prefix_dash . 'query-post-type']) ?
												stripcslashes(sanitize_text_field($_POST[self::$prefix_dash . 'query-post-type'])) :
												self::$default['query']['post_type'];

			// Taxonomies

			foreach ($taxonomies as $taxonomy) {

				$settings['query']['taxonomy'][$taxonomy]['exclude'] = isset($_POST[self::$prefix_dash . 'query-exclude-' . $taxonomy]) ?
													       str_replace(' ', '', rtrim(sanitize_text_field($_POST[self::$prefix_dash . 'query-exclude-' . $taxonomy]), ',')) : '';
			}

			// Save settings to database

			update_option(self::$prefix . 'settings', $settings);

			// Clear all transient data when the user saves settings

			global $wpdb;

			$query = $wpdb->query($wpdb->prepare( "DELETE FROM {$wpdb->options} WHERE option_name LIKE %s", '%' . $wpdb->esc_like(self::$prefix . 'cached_posts_') . '%'));
		}

		require('admin/views/settings.php');
	}
}